@@ -23,7 +23,7 @@ import asyncio
 import logging
 
 from ..compat import map
-from ..exceptions import TransportError
+from ..exceptions import NotFoundError, TransportError
 from ..helpers.actions import (
     _ActionChunker,
     _process_bulk_chunk_error,
@@ -59,25 +59,33 @@ async def _process_bulk_chunk(
     bulk_data,
     raise_on_exception=True,
     raise_on_error=True,
+    ignore_status=(),
     *args,
     **kwargs
 ):
     """
     Send a bulk request to elasticsearch and process the output.
     """
+    if not isinstance(ignore_status, (list, tuple)):
+        ignore_status = (ignore_status,)
+
     try:
         # send the actual request
-        resp = await client.bulk("\n".join(bulk_actions) + "\n", *args, **kwargs)
+        resp = await client.bulk(*args, body="\n".join(bulk_actions) + "\n", **kwargs)
     except TransportError as e:
         gen = _process_bulk_chunk_error(
             error=e,
             bulk_data=bulk_data,
+            ignore_status=ignore_status,
             raise_on_exception=raise_on_exception,
             raise_on_error=raise_on_error,
         )
     else:
         gen = _process_bulk_chunk_success(
-            resp=resp, bulk_data=bulk_data, raise_on_error=raise_on_error
+            resp=resp,
+            bulk_data=bulk_data,
+            ignore_status=ignore_status,
+            raise_on_error=raise_on_error,
         )
     for item in gen:
         yield item
@@ -121,6 +129,7 @@ async def async_streaming_bulk(
     initial_backoff=2,
     max_backoff=600,
     yield_ok=True,
+    ignore_status=(),
     *args,
     **kwargs
 ):
@@ -156,6 +165,7 @@ async def async_streaming_bulk(
         2**retry_number``
     :arg max_backoff: maximum number of seconds a retry will wait
     :arg yield_ok: if set to False will skip successful documents in the output
+    :arg ignore_status: list of HTTP status code that you want to ignore
     """
 
     async def map_actions():
@@ -182,8 +192,9 @@ async def async_streaming_bulk(
                         bulk_data,
                         raise_on_exception,
                         raise_on_error,
+                        ignore_status,
                         *args,
-                        **kwargs
+                        **kwargs,
                     ),
                 ):
 
@@ -218,7 +229,9 @@ async def async_streaming_bulk(
                 bulk_actions, bulk_data = to_retry, to_retry_data
 
 
-async def async_bulk(client, actions, stats_only=False, *args, **kwargs):
+async def async_bulk(
+    client, actions, stats_only=False, ignore_status=(), *args, **kwargs
+):
     """
     Helper for the :meth:`~elasticsearch.AsyncElasticsearch.bulk` api that provides
     a more human friendly interface - it consumes an iterator of actions and
@@ -240,6 +253,7 @@ async def async_bulk(client, actions, stats_only=False, *args, **kwargs):
     :arg actions: iterator containing the actions
     :arg stats_only: if `True` only report number of successful/failed
         operations instead of just number of successful and a list of error responses
+    :arg ignore_status: list of HTTP status code that you want to ignore
 
     Any additional keyword arguments will be passed to
     :func:`~elasticsearch.helpers.async_streaming_bulk` which is used to execute
@@ -253,7 +267,9 @@ async def async_bulk(client, actions, stats_only=False, *args, **kwargs):
 
     # make streaming_bulk yield successful results so we can count them
     kwargs["yield_ok"] = True
-    async for ok, item in async_streaming_bulk(client, actions, *args, **kwargs):
+    async for ok, item in async_streaming_bulk(
+        client, actions, ignore_status=ignore_status, *args, **kwargs
+    ):
         # go through request-response pairs and detect failures
         if not ok:
             if not stats_only:
@@ -322,9 +338,26 @@ async def async_scan(
         query = query.copy() if query else {}
         query["sort"] = "_doc"
 
+    # Grab options that should be propagated to every
+    # API call within this helper instead of just 'search()'
+    transport_kwargs = {}
+    for key in ("headers", "api_key", "http_auth"):
+        if key in kwargs:
+            transport_kwargs[key] = kwargs[key]
+
+    # If the user is using 'scroll_kwargs' we want
+    # to propagate there too, but to not break backwards
+    # compatibility we'll not override anything already given.
+    if scroll_kwargs is not None and transport_kwargs:
+        for key, val in transport_kwargs.items():
+            scroll_kwargs.setdefault(key, val)
+
     # initial search
+    search_kwargs = kwargs.copy()
+    if query:
+        search_kwargs.update(query)
     resp = await client.search(
-        body=query, scroll=scroll, size=size, request_timeout=request_timeout, **kwargs
+        scroll=scroll, size=size, request_timeout=request_timeout, **search_kwargs
     )
     scroll_id = resp.get("_scroll_id")
 
@@ -358,14 +391,15 @@ async def async_scan(
                         ),
                     )
             resp = await client.scroll(
-                body={"scroll_id": scroll_id, "scroll": scroll}, **scroll_kwargs
+                scroll_id=scroll_id, scroll=scroll, **scroll_kwargs
             )
             scroll_id = resp.get("_scroll_id")
 
     finally:
         if scroll_id and clear_scroll:
             await client.clear_scroll(
-                body={"scroll_id": [scroll_id]},
+                scroll_id=scroll_id,
+                **transport_kwargs,
                 ignore=(404,),
                 params={"__elastic_client_meta": (("h", "s"),)},
             )
@@ -379,6 +413,7 @@ async def async_reindex(
     target_client=None,
     chunk_size=500,
     scroll="5m",
+    op_type=None,
     scan_kwargs={},
     bulk_kwargs={},
 ):
@@ -408,6 +443,9 @@ async def async_reindex(
     :arg chunk_size: number of docs in one chunk sent to es (default: 500)
     :arg scroll: Specify how long a consistent view of the index should be
         maintained for scrolled search
+    :arg op_type: Explicit operation type. Defaults to '_index'. Data streams must
+        be set to 'create'. If not specified, will auto-detect if target_index is a
+        data stream.
     :arg scan_kwargs: additional kwargs to be passed to
         :func:`~elasticsearch.helpers.async_scan`
     :arg bulk_kwargs: additional kwargs to be passed to
@@ -418,18 +456,41 @@ async def async_reindex(
         client, query=query, index=source_index, scroll=scroll, **scan_kwargs
     )
 
-    async def _change_doc_index(hits, index):
+    async def _change_doc_index(hits, index, op_type):
         async for h in hits:
             h["_index"] = index
+            if op_type is not None:
+                h["_op_type"] = op_type
             if "fields" in h:
                 h.update(h.pop("fields"))
             yield h
 
     kwargs = {"stats_only": True}
     kwargs.update(bulk_kwargs)
+
+    is_data_stream = False
+    try:
+        # Verify if the target_index is data stream or index
+        data_streams = await target_client.indices.get_data_stream(
+            target_index, expand_wildcards="all"
+        )
+        is_data_stream = any(
+            data_stream["name"] == target_index
+            for data_stream in data_streams["data_streams"]
+        )
+    except (TransportError, KeyError, NotFoundError):
+        # If its not data stream, might be index
+        pass
+
+    if is_data_stream:
+        if op_type not in (None, "create"):
+            raise ValueError("Data streams must have 'op_type' set to 'create'")
+        else:
+            op_type = "create"
+
     return await async_bulk(
         target_client,
-        _change_doc_index(docs, target_index),
+        _change_doc_index(docs, target_index, op_type),
         chunk_size=chunk_size,
-        **kwargs
+        **kwargs,
     )
